@@ -147,7 +147,7 @@ const NetworkSecretDialog = new Lang.Class({
 
         this._okButton = { label:  _("Connect"),
                            action: Lang.bind(this, this._onOk),
-                           key:    Clutter.KEY_Return,
+                           default: true
                          };
 
         this.setButtons([{ label: _("Cancel"),
@@ -165,11 +165,6 @@ const NetworkSecretDialog = new Lang.Class({
         }
 
         this._okButton.button.reactive = valid;
-        this._okButton.button.can_focus = valid;
-        if (valid)
-            this._okButton.button.remove_style_pseudo_class('disabled');
-        else
-            this._okButton.button.add_style_pseudo_class('disabled');
     },
 
     _onOk: function() {
