@@ -134,12 +134,7 @@ const PopupBaseMenuItem = new Lang.Class({
 
         this.sensitive = sensitive;
         this.actor.reactive = sensitive;
-        this.actor.can_focus = sensitive;
 
-        if (sensitive)
-            this.actor.remove_style_pseudo_class('insensitive');
-        else
-            this.actor.add_style_pseudo_class('insensitive');
         this.emit('sensitive-changed', sensitive);
     },
 
@@ -773,12 +768,10 @@ const PopupSwitchMenuItem = new Lang.Class({
             this._statusLabel.text = text;
             this._statusBin.child = this._statusLabel;
             this.actor.reactive = false;
-            this.actor.can_focus = false;
             this.actor.accessible_role = Atk.Role.MENU_ITEM;
         } else {
             this._statusBin.child = this._switch.actor;
             this.actor.reactive = true;
-            this.actor.can_focus = true;
             this.actor.accessible_role = Atk.Role.CHECK_MENU_ITEM;
         }
         this.checkAccessibleState();
@@ -875,6 +868,7 @@ const PopupMenuBase = new Lang.Class({
 
         this._activeMenuItem = null;
         this._childMenus = [];
+        this._settingsActions = { };
     },
 
     addAction: function(title, callback) {
@@ -902,9 +896,19 @@ const PopupMenuBase = new Lang.Class({
                            Main.overview.hide();
                            app.activate();
                        });
+
+        this._settingsActions[desktopFile] = menuItem;
+
         return menuItem;
     },
 
+    setSettingsVisibility: function(visible) {
+        for (let id in this._settingsActions) {
+            let item = this._settingsActions[id];
+            item.actor.visible = visible;
+        }
+    },
+
     isEmpty: function() {
         return this.box.get_n_children() == 0;
     },
@@ -1891,11 +1895,7 @@ const RemoteMenu = new Lang.Class({
             }));
         }
 
-        item.actor.reactive = item.actor.can_focus = action.enabled;
-        if (action.enabled)
-            item.actor.remove_style_pseudo_class('insensitive');
-        else
-            item.actor.add_style_pseudo_class('insensitive');
+        item.actor.reactive = action.enabled;
 
         destroyId = item.connect('destroy', Lang.bind(this, function() {
             item.disconnect(destroyId);
@@ -2027,12 +2027,7 @@ const RemoteMenu = new Lang.Class({
         if (action.items.length) {
             for (let i = 0; i < action.items.length; i++) {
                 let item = action.items[i];
-                item.actor.reactive = item.actor.can_focus = action.enabled;
-
-                if (action.enabled)
-                    item.actor.remove_style_pseudo_class('insensitive');
-                else
-                    item.actor.add_style_pseudo_class('insensitive');
+                item.actor.reactive = action.enabled;
             }
         }
     }
