@@ -1,12 +1,9 @@
 // -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-
-const ByteArray = imports.byteArray;
 const GLib = imports.gi.GLib;
 const GObject = imports.gi.GObject;
 const Lang = imports.lang;
-const Mainloop = imports.mainloop;
 const NetworkManager = imports.gi.NetworkManager;
 const NMClient = imports.gi.NMClient;
-const Shell = imports.gi.Shell;
 const Signals = imports.signals;
 const St = imports.gi.St;
 
@@ -1698,6 +1695,15 @@ const NMApplet = new Lang.Class({
         this._primaryIcon.icon_name = iconName;
     },
 
+    setLockedState: function(locked) {
+        // FIXME: more design discussion is needed before we can
+        // expose part of this menu
+
+        if (locked)
+            this.menu.close();
+        this.actor.reactive = !locked;
+    },
+
     _ensureSource: function() {
         if (!this._source) {
             this._source = new MessageTray.Source(_("Network Manager"),
