@@ -482,9 +482,9 @@ const ChatSource = new Lang.Class({
         this._notification.appendAliasChange(oldAlias, newAlias);
     },
 
-    createNotificationIcon: function() {
+    createIcon: function(size) {
         this._iconBox = new St.Bin({ style_class: 'avatar-box' });
-        this._iconBox._size = this.ICON_SIZE;
+        this._iconBox._size = size;
         let textureCache = St.TextureCache.get_default();
         let file = this._contact.get_avatar_file();
 
@@ -532,8 +532,8 @@ const ChatSource = new Lang.Class({
     },
 
     _updateAvatarIcon: function() {
-        this._setSummaryIcon(this.createNotificationIcon());
-        this._notification.update(this._notification.title, null, { customContent: true, icon: this.createNotificationIcon() });
+        this._setSummaryIcon(this.createIcon(this.ICON_SIZE));
+        this._notification.update(this._notification.title, null, { customContent: true });
     },
 
     open: function(notification) {
@@ -578,7 +578,7 @@ const ChatSource = new Lang.Class({
             this._pendingMessages.push(message);
         }
 
-        this._updateCount();
+        this.countUpdated();
 
         let showTimestamp = false;
 
@@ -624,8 +624,17 @@ const ChatSource = new Lang.Class({
         this.destroy();
     },
 
-    _updateCount: function() {
-        this._setCount(this._pendingMessages.length, this._pendingMessages.length > 0);
+    /* All messages are new messages for Telepathy sources */
+    get count() {
+        return this._pendingMessages.length;
+    },
+
+    get unseenCount() {
+        return this.count;
+    },
+
+    get countVisible() {
+        return this.count > 0;
     },
 
     _messageReceived: function(channel, message) {
@@ -633,7 +642,7 @@ const ChatSource = new Lang.Class({
             return;
 
         this._pendingMessages.push(message);
-        this._updateCount();
+        this.countUpdated();
 
         message = makeMessageFromTpMessage(message, NotificationDirection.RECEIVED);
         this._notification.appendMessage(message);
@@ -710,7 +719,7 @@ const ChatSource = new Lang.Class({
 
         if (idx >= 0) {
             this._pendingMessages.splice(idx, 1);
-            this._updateCount();
+            this.countUpdated();
         }
         else
             throw new Error('Message not in our pending list: ' + message);
@@ -1031,9 +1040,9 @@ const ApproverSource = new Lang.Class({
         this.parent();
     },
 
-    createNotificationIcon: function() {
+    createIcon: function(size) {
         return new St.Icon({ gicon: this._gicon,
-                             icon_size: this.ICON_SIZE });
+                             icon_size: size });
     }
 });
 
