@@ -16,7 +16,6 @@ const GnomeSession = imports.misc.gnomeSession;
 const Main = imports.ui.main;
 const PanelMenu = imports.ui.panelMenu;
 const PopupMenu = imports.ui.popupMenu;
-const ScreenSaver = imports.misc.screenSaver;
 const Util = imports.misc.util;
 
 const LOCKDOWN_SCHEMA = 'org.gnome.desktop.lockdown';
@@ -41,6 +40,34 @@ const IMStatus = {
 // Copyright (C) 2004-2005 James M. Cape <jcape@ignore-your.tv>.
 // Copyright (C) 2008,2009 Red Hat, Inc.
 
+const UserAvatarWidget = new Lang.Class({
+    Name: 'UserAvatarWidget',
+
+    _init: function(user) {
+        this._user = user;
+
+        this.actor = new St.Bin({ style_class: 'status-chooser-user-icon',
+                                  track_hover: true,
+                                  reactive: true });
+    },
+
+    update: function() {
+        let iconFile = this._user.get_icon_file();
+        if (!GLib.file_test(iconFile, GLib.FileTest.EXISTS))
+            iconFile = null;
+
+        if (iconFile) {
+            let file = Gio.File.new_for_path(iconFile);
+            this.actor.child = null;
+            this.actor.style = 'background-image: url("%s");'.format(iconFile);
+        } else {
+            this.actor.style = null;
+            this.actor.child = new St.Icon({ icon_name: 'avatar-default',
+                                             icon_type: St.IconType.SYMBOLIC,
+                                             icon_size: DIALOG_ICON_SIZE });
+        }
+    }
+});
 
 const IMStatusItem = new Lang.Class({
     Name: 'IMStatusItem',
@@ -108,7 +135,11 @@ const IMStatusChooserItem = new Lang.Class({
         this.parent({ reactive: false,
                       style_class: 'status-chooser' });
 
-        this._iconBin = new St.Button({ style_class: 'status-chooser-user-icon' });
+        this._userManager = AccountsService.UserManager.get_default();
+        this._user = this._userManager.get_user(GLib.get_user_name());
+
+        this._avatar = new UserAvatarWidget(this._user);
+        this._iconBin = new St.Button({ child: this._avatar.actor });
         this.addActor(this._iconBin);
 
         this._iconBin.connect('clicked', Lang.bind(this,
@@ -186,10 +217,6 @@ const IMStatusChooserItem = new Lang.Class({
                 }
             }));
 
-        this._userManager = AccountsService.UserManager.get_default();
-
-        this._user = this._userManager.get_user(GLib.get_user_name());
-
         this._userLoadedId = this._user.connect('notify::is-loaded',
                                                 Lang.bind(this,
                                                           this._updateUser));
@@ -227,44 +254,12 @@ const IMStatusChooserItem = new Lang.Class({
     },
 
     _updateUser: function() {
-        let iconFile = null;
-        if (this._user.is_loaded) {
+        if (this._user.is_loaded)
             this._name.label.set_text(this._user.get_real_name());
-            iconFile = this._user.get_icon_file();
-            if (!GLib.file_test(iconFile, GLib.FileTest.EXISTS))
-                iconFile = null;
-        } else {
-            this._name.label.set_text("");
-        }
-
-        if (iconFile)
-            this._setIconFromFile(iconFile);
         else
-            this._setIconFromName('avatar-default');
-    },
+            this._name.label.set_text("");
 
-    _setIconFromFile: function(iconFile) {
-        this._iconBin.set_style('background-image: url("' + iconFile + '");' +
-                                'background-size: contain;');
-        this._iconBin.child = null;
-    },
-
-    _setIconFromName: function(iconName) {
-        this._iconBin.set_style(null);
-
-        if (iconName != null) {
-            let textureCache = St.TextureCache.get_default();
-            let icon = textureCache.load_icon_name(this._iconBin.get_theme_node(),
-                                                   iconName,
-                                                   St.IconType.SYMBOLIC,
-                                                   DIALOG_ICON_SIZE);
-
-            this._iconBin.child = icon;
-            this._iconBin.show();
-        } else {
-            this._iconBin.child = null;
-            this._iconBin.hide();
-        }
+        this._avatar.update();
     },
 
     _statusForPresence: function(presence) {
@@ -455,7 +450,6 @@ const UserMenuButton = new Lang.Class({
         this._accountMgr = Tp.AccountManager.dup();
 
         this._upClient = new UPowerGlib.Client();
-        this._screenSaverProxy = new ScreenSaver.ScreenSaverProxy();
         this.actor.connect('destroy', Lang.bind(this, this._onDestroy));
 
         this._iconBox = new St.Bin();
@@ -542,6 +536,12 @@ const UserMenuButton = new Lang.Class({
         this._upClient.connect('notify::can-suspend', Lang.bind(this, this._updateSuspendOrPowerOff));
     },
 
+    setLockedState: function(locked) {
+        if (locked)
+            this.menu.close();
+        this.actor.reactive = !locked;
+    },
+
     _onDestroy: function() {
         this._user.disconnect(this._userLoadedId);
         this._user.disconnect(this._userChangedId);
@@ -654,8 +654,10 @@ const UserMenuButton = new Lang.Class({
     },
 
     _onAccountRemoved: function(accountMgr, account) {
-        account.disconnect(account._changingId);
-        account._changingId = 0;
+        if (account._changingId) {
+            account.disconnect(account._changingId);
+            account._changingId = 0;
+        }
         this._updateChangingPresence();
     },
 
@@ -767,16 +769,13 @@ const UserMenuButton = new Lang.Class({
 
     _onLockScreenActivate: function() {
         Main.overview.hide();
-        this._screenSaverProxy.LockRemote();
+        Main.screenShield.lock(true);
     },
 
     _onLoginScreenActivate: function() {
         Main.overview.hide();
-        // Ensure we only move to GDM after the screensaver has activated; in some
-        // OS configurations, the X server may block event processing on VT switch
-        this._screenSaverProxy.SetActiveRemote(true, Lang.bind(this, function() {
-            this._userManager.goto_login_session();
-        }));
+        Main.screenShield.lock(false);
+        this._userManager.goto_login_session();
     },
 
     _onQuitSessionActivate: function() {
@@ -798,10 +797,13 @@ const UserMenuButton = new Lang.Class({
             this._suspendOrPowerOffItem.state == PopupMenu.PopupAlternatingMenuItemState.DEFAULT) {
             this._session.ShutdownRemote();
         } else {
-            // Ensure we only suspend after locking the screen
-            this._screenSaverProxy.LockRemote(Lang.bind(this, function() {
+            let tmpId = Main.screenShield.connect('lock-screen-shown', Lang.bind(this, function() {
+                Main.screenShield.disconnect(tmpId);
+
                 this._upClient.suspend_sync(null);
             }));
+
+            Main.screenShield.lock(true);
         }
     }
 });
