@@ -591,6 +591,8 @@ stop_scrolling (StScrollBar *bar)
   if (!bar->priv->capture_handler)
     return;
 
+  st_widget_remove_style_pseudo_class (ST_WIDGET (bar->priv->handle), "active");
+
   stage = CLUTTER_STAGE (clutter_actor_get_stage (bar->priv->trough));
   g_signal_handler_disconnect (stage, bar->priv->capture_handler);
   bar->priv->capture_handler = 0;
@@ -651,6 +653,8 @@ handle_button_press_event_cb (ClutterActor       *actor,
                                             &priv->y_origin))
     return FALSE;
 
+  st_widget_add_style_pseudo_class (ST_WIDGET (priv->handle), "active");
+
   /* Account for the scrollbar-trough-handle nesting. */
   priv->x_origin += clutter_actor_get_x (priv->trough);
   priv->y_origin += clutter_actor_get_y (priv->trough);
