@@ -1404,6 +1404,17 @@ st_widget_name_notify (StWidget   *widget,
   st_widget_style_changed (widget);
 }
 
+static void
+st_widget_reactive_notify (StWidget   *widget,
+                           GParamSpec *pspec,
+                           gpointer    data)
+{
+  if (clutter_actor_get_reactive (CLUTTER_ACTOR (widget)))
+    st_widget_remove_style_pseudo_class (widget, "insensitive");
+  else
+    st_widget_add_style_pseudo_class (widget, "insensitive");
+}
+
 static void
 st_widget_first_child_notify (StWidget   *widget,
                               GParamSpec *pspec,
@@ -1466,6 +1477,7 @@ st_widget_init (StWidget *actor)
 
   /* connect style changed */
   g_signal_connect (actor, "notify::name", G_CALLBACK (st_widget_name_notify), NULL);
+  g_signal_connect (actor, "notify::reactive", G_CALLBACK (st_widget_reactive_notify), NULL);
 
   g_signal_connect (actor, "notify::first-child", G_CALLBACK (st_widget_first_child_notify), NULL);
   g_signal_connect (actor, "notify::last-child", G_CALLBACK (st_widget_last_child_notify), NULL);
@@ -1880,7 +1892,8 @@ st_widget_real_navigate_focus (StWidget         *widget,
   while (focus_child && clutter_actor_get_parent (focus_child) != widget_actor)
     focus_child = clutter_actor_get_parent (focus_child);
 
-  if (widget->priv->can_focus)
+  if (clutter_actor_get_reactive (widget_actor) &&
+      widget->priv->can_focus)
     {
       if (!focus_child)
         {
